
#ifndef OSFCN_H
#define OSFCN_H 1

#include <std.h>
#include <time.h>
#include <sys/types.h>
#if _G_HAVE_SYS_SOCKET
#include <sys/socket.h>
#endif
#include <sys/time.h>
#if _G_HAVE_SYS_RESOURCE
#include <sys/resource.h>
#endif


#endif
