in word set if
in word set do
in word set int
in word set for
in word set case
in word set char
in word set auto
in word set goto
in word set else
in word set long
in word set void
in word set enum
in word set float
in word set short
in word set union
in word set break
in word set while
in word set const
in word set double
in word set static
in word set extern
in word set struct
in word set return
in word set sizeof
in word set switch
in word set signed
in word set typedef
in word set default
in word set unsigned
in word set continue
in word set register
in word set volatile
