/* This file is generated by hand, since the shell script will not run
 * under VMS.  I hope that someday something more automatic can be created.
 */	
#ifndef _G_config_h
#define _G_config_h
#define _G_NAMES_HAVE_UNDERSCORE 1
#define _G_DOLLAR_IN_LABEL 0
#define _G_HAVE_ST_BLKSIZE 0
#define _G_clock_t long
#define _G_dev_t char *
#define _G_fpos_t long
#define _G_gid_t long
#define _G_ino_t unsigned short
#define _G_mode_t unsigned long
#define _G_nlink_t unsigned long
#define _G_off_t long
#define _G_pid_t long
#define _G_ptrdiff_t int
#define _G_size_t unsigned int
#define _G_time_t long int
#define _G_uid_t unsigned short
#define _G_wchar_t long int
#define _G_ssize_t  long /* deduced */
/* #define _G_NEED_STDARG_H */
#define _G_va_list char* /* default */
#define _G_signal_return_type int
#define _G_sprintf_return_type int
#define _G_BUFSIZ  512
#define _G_FOPEN_MAX 32 /* default */
#define _G_FILENAME_MAX  1024 /* default */
#define _G_NULL 0 /* default */
#ifdef _G_USE_PROTOS
#define _G_ARGS(ARGLIST) ARGLIST
#else
#define _G_ARGS(ARGLIST) (...)
#endif
/* #define _G_SYSV */
#define _G_HAVE_SYS_RESOURCE 1
#define _G_HAVE_SYS_SOCKET 1
#define _G_HAVE_UNISTD 0
#define _G_HAVE_DIRENT 0
#define _G_BROKEN_SIGNED_CHAR
#define _G_FRIEND_BUG
#endif /* !_G_config_h */
